# frozen_string_literal: true

require 'bundler'
Bundler.setup(:default)
require 'rack'

# Define `Wouter` so we can load stuff
class Wouter; end

require_relative 'wouter/util'
require_relative 'wouter/route'
require_relative 'wouter/endpoint'
require_relative 'wouter/wrapper'

# Main route definition DSL
class Wouter
  class <<self
    def get(route, klass)
      define_route(:GET, route, klass)
    end

    def put(route, klass)
      define_route(:PUT, route, klass)
    end

    def post(route, klass)
      define_route(:POST, route, klass)
    end

    def delete(route, klass)
      define_route(:DELETE, route, klass)
    end

    def middleware(klass, *args, &block)
      app.use(klass, *args, &block)
    end

    def build
      app.run(Wrapper.new(routes: routes, config: config))
      app.to_app
    end

    private

    def app
      @app ||= Rack::Builder.new
    end

    def routes
      @routes ||= []
    end

    def config
      @config ||= {}
    end

    def define_route(request_method, path, klass)
      routes << Wouter::Route.new(request_method: request_method, path: path, klass: klass)
    end
  end
end
