# frozen_string_literal: true

# `Wouter::Endpoint` is a class to help define Rack compatible endpoints that
# can be assigned to a route path. Each `Wouter::Endpoint` instance is a Rack
# application of it's own. It provides convience methods for creating endpoints.
#
class Wouter::Endpoint
  class NotImplementedError < StandardError; end

  attr_reader :req, :res, :config

  def initialize(req, res)
    @req = req
    @res = res
    @config = req.env['wouter']
  end

  def self.call(env, config = {})
    endpoint = new(Rack::Request.new(env), Rack::Response.new)
    response = endpoint.respond

    return response.finish if response.is_a?(Rack::Response)

    endpoint.res.write response
    endpoint.res.finish
  end

  def respond
    raise NotImplementedError, 'you must implemented #respond in your Wouter::Endpoint class'
  end

  def params
    @req.params
  end

  def headers(hash = nil)
    @res.headers.merge! hash if hash
    @res.headers
  end

  def status(code)
    @res.status = code
    @res.status
  end

  def not_found
    Wouter::Util.not_found
  end

  def redirect(location)
    status 302
    headers['Location'] = location
    @res
  end
end
