# frozen_string_literal: true

# `Wouter::Wrapper` is a class that provides a top level Rack application containing
# all the routing endpoints. HTTP requests come through `Wouter::Wrapper` and it dispatches
# to the appropriate route class that was defined by the user.
#
class Wouter::Wrapper
  def initialize(routes:, config: {})
    @routes = routes
    @config = config
  end

  def call(env)
    env['wouter'] = @config
    request = Rack::Request.new(env)
    route = find_route(request)

    if route
      route.params(request).each do |name, value|
        request.update_param(name, value)
      end
      route.klass.call(request.env)
    else
      Wouter::Util.not_found
    end
  end

  private

  def find_route(request)
    @routes.find do |route|
      route.match?(request)
    end
  end
end
